html {
  background: #faf9f4;
}
body {
  background: transparent;
  top: 0;
  bottom:0;
  left:0;
  right:0;
}
.logo {
  padding-top: 30px;
  h1 {
      font-size: 60px;
      small {
        font-size: 30px;

      }
  }
}

*, body, html, .article, h1, h2, h3, h4, h5, h6, p, div, span, strong, a, li, ul, ol, nav {
  font-family: "Montserrat";
}

.httpretty-navbar {
  .uk-navbar-brand{
    font-size: 28px;
    padding-top: 4px;
    small {
      font-size:14px;
    }
  }
}

a.uk-button-primary {
  color:white;
}